package mvd.jester.simulator.events;

import java.util.Comparator;
import java.util.PriorityQueue;
import mvd.jester.priority.PriorityManager;

public class EventQueue extends PriorityQueue<IEvent> {

    /**
    *
    */
    private static final long serialVersionUID = -6221675577222053574L;

    public EventQueue(final PriorityManager priorityManager) {
        super(new EventComparator(priorityManager));
    }

    private static class EventComparator implements Comparator<IEvent> {

        private final PriorityManager priorityManager;

        private EventComparator(final PriorityManager priorityManager) {
            this.priorityManager = priorityManager;
        }

        /**
         * Compare two events. The following comparisons are made: 1. time of the events 2. priority
         * of the events (according to the event enumeration order) 3. priority of the jobs
         * (according to the scheduling priority policy)
         *
         * @param e1 The first event.
         * @param e2 The second event.
         * @return 0 if events are considered equivalent, negative number if the first event is
         *         prior to the second one and positive number if the first event is after the
         *         second one.
         */
        @Override
        public int compare(final IEvent e1, final IEvent e2) {
            final long timeDiff = e1.getTime() - e2.getTime();
            if (timeDiff != 0) {
                return (int) timeDiff;
            }
            final int ordDiff = e1.getType().ordinal() - e2.getType().ordinal();
            if (ordDiff != 0) {
                return ordDiff;
            }
            if (!(e1 instanceof ISubtaskEvent && e2 instanceof ISubtaskEvent)) {
                return 0;
            }
            return priorityManager.compare(((ISubtaskEvent) e1).getJobContext(),
                    ((ISubtaskEvent) e2).getJobContext());
        }

    }
}
