package mvd.jester.simulator.model;

import java.util.HashSet;
import java.util.Set;
import org.jgrapht.experimental.dag.DirectedAcyclicGraph;
import org.jgrapht.graph.DefaultEdge;
import mvd.jester.model.DagTask;
import mvd.jester.model.Subtask;

public class JobContext {

    private final DagTask task;
    private final long releaseTime;
    private final long deadline;
    private final Subtask source;
    private final Set<ExtendedSubtaskContext> finishedSubtasks;
    // private long activeThreads; // TODO: Use threads

    public JobContext(DagTask task, long releaseTime) {
        this.task = task;
        this.releaseTime = releaseTime;
        this.deadline = releaseTime + task.getDeadline();
        this.source = findSource(task);
        this.finishedSubtasks = new HashSet<>();
        // this.activeThreads = task.getNumberOfThreads();
    }

    private Subtask findSource(DagTask task) {
        DirectedAcyclicGraph<Subtask, DefaultEdge> jobDag = task.getJobDag();
        for (Subtask subtask : jobDag) {
            if (jobDag.inDegreeOf(subtask) == 0) {
                return subtask;
            }
        }
        throw new RuntimeException("No source found in DAG. How can that be!?");
    }

    /**
     * @return the task
     */
    public DagTask getTask() {
        return task;
    }

    /**
     * @return the source
     */
    public Subtask getSource() {
        return source;
    }

    /**
     * @return the deadline
     */
    public long getDeadline() {
        return deadline;
    }

    /**
     * @return the releaseTime
     */
    public long getReleaseTime() {
        return releaseTime;
    }

    /**
     * @return the finishedSubtasks
     */
    public Set<ExtendedSubtaskContext> getFinishedSubtasks() {
        return finishedSubtasks;
    }

    public void registerFinishedSubtask(ExtendedSubtaskContext subtaskContext) {
        if (!finishedSubtasks.add(subtaskContext)) {
            throw new RuntimeException(
                    "Subtask could not be added to finished subtask which means it was executed twice!");
        }
    }

    public boolean checkRemainingThreads() {
        return true; // TODO: implement check in future
    }

    public String toString() {
        return "of task " + getTask();
    }

}
