package mvd.jester.utils;


public class BinaryDecompositionTree<N> {

    private Node<N> root;

    public BinaryDecompositionTree() {
    }

    public Node<N> getRootNode() {
        return root;
    }

    public boolean contains(final N object) {
        if (root == null) {
            return false;
        }
        return root.contains(object);
    }

    public boolean isEmpty() {
        return root == null;
    }

    public void setRoot(final Node<N> root) {
        this.root = root;
    }

    public static class Node<T> {
        private final Node<T> parentNode;
        private Node<T> leftNode;
        private Node<T> rightNode;
        private final NodeType nodeType;
        private final T object;

        public Node(final Node<T> parentNode, final NodeType nodeType) {
            this.parentNode = parentNode;
            this.nodeType = nodeType;
            this.object = null;
        }

        public boolean contains(final T object) {
            if (nodeType.equals(NodeType.LEAF)) {
                return this.object == object;
            } else {
                boolean leftContains = false;
                boolean rightContains = false;
                if (leftNode != null) {
                    leftContains = leftNode.contains(object);
                }
                if (rightNode != null) {
                    rightContains = rightNode.contains(object);
                }
                return leftContains || rightContains;
            }
        }

        public Node(final Node<T> parentNode, final T object) {
            this.parentNode = parentNode;
            this.nodeType = NodeType.LEAF;
            this.object = object;
        }

        /**
         * @return the parentNode
         */
        public Node<T> getParentNode() {
            return parentNode;
        }

        public NodeType getNodeType() {
            return nodeType;
        }

        public T getObject() {
            return object;
        }

        /**
         * @param leftNode the leftNode to set
         */
        public Node<T> setLeftNode(final Node<T> leftNode) {
            this.leftNode = leftNode;
            return this.leftNode;
        }

        /**
         * @return the leftNode
         */
        public Node<T> getLeftNode() {
            return leftNode;
        }

        /**
         * @param rightNode the rightNode to set
         */
        public Node<T> setRightNode(final Node<T> rightNode) {
            this.rightNode = rightNode;
            return this.rightNode;
        }

        /**
         * @return the rightNode
         */
        public Node<T> getRightNode() {
            return rightNode;
        }
    }

    public enum NodeType {
        SEQ, PAR, LEAF
    }
}
